<?php
/*
Plugin Name: Piwishadow
Version: 0.1.9
Description: Display pictures, videos and piwigo pages in modal overlay window.
Plugin URI: http://piwigo.org/ext/extension_view.php?eid=521
Author: Binaryworld
Author URI: http://binaryworld.hd.free.fr/
*/
if (!defined('PHPWG_ROOT_PATH')) die('Hacking attempt!');
define('PIWISHADOW_PATH' , PHPWG_PLUGINS_PATH . basename(dirname(__FILE__)) . '/');
if (!defined( 'DS')) define( 'DS', DIRECTORY_SEPARATOR );
load_language('plugin.lang', PIWISHADOW_PATH);

include_once PIWISHADOW_PATH .'overlaylibs.inc.php';
include_once PIWISHADOW_PATH .'config.inc.php';

class Piwishadow {

	public static $PLUGIN_ID = NULL;
	private static $_instance = NULL;
	
	private $_config;
	private $_pluginPath;
	private $_pluginURL;
	private $_overlayLibs = array();

    /**
     * Prevent direct object creation
     */
    final private function  __construct() { }

    /**
     * Prevent object cloning
     */
    final private function  __clone() { }

    final public static function getInstance($pluginId){
        if(null !== self::$_instance){
            return self::$_instance;
        }
        $oldInstance = get_plugin_data($pluginId);
        
        self::$_instance = new Piwishadow();
		self::$PLUGIN_ID = $pluginId;
		//self::$_instance->loadConfig();
		set_plugin_data(self::$PLUGIN_ID, self::$_instance);
        return self::$_instance;
    }	

	
	public function getPluginPath() {
		if ( !isset($this->_pluginPath) ) $this->_pluginPath = dirname(__FILE__);
		return $this->_pluginPath;
	}
	public function getPluginURL() {
		if ( !isset($this->_pluginURL) ) $this->_pluginURL =  get_root_url() .'plugins/'. basename(dirname(__FILE__));
		return $this->_pluginURL;
	}
	
	public function &getConfig() {
		if (isset($this->_config)) {
			return $this->_config;
		}
		if ( ($config = $this->loadConfig()) != false ){
			$this->_config = $config;
			return $this->_config;
		}
		return false;
	}
	
	
	public function getDefaultConfig() {
		return Config::createDefaultConfig($this->getPluginPath() .DS. Config::$DEFAULT_CONFIG_FILENAME);
	}
	
    public function loadConfig() {
    	$filename = $this->getPluginPath() .DS. Config::$CONFIG_FILENAME;
    	if (file_exists($filename) && Config::read($filename, $c)) {
   			//$this->_config = new Config( $config );
    		return new Config( $c );
    	} else {
    		//$this->_config = $this->getDefaultConfig();
    		//$this->saveConfig();
    		return $this->getDefaultConfig();
    	}
    }
    public function saveConfig()  {
    	$filename = $this->getPluginPath() .DS. Config::$CONFIG_FILENAME;
    	$this->getConfig()->save($filename);
    }
    
    //
    // Modal overlay lib
    //
    public function getOverlayLib($libName = null) {
    	if (empty($libName)) 
    		$libName = $this->getConfig()->get('modalOverlayLib', 'overlaylib') ;
    	
    	if ( !empty( $this->_overlayLibs[$libName] )) {
			return	$this->_overlayLibs[$libName];		
    	}
    	$this->_overlayLibs[$libName] = ModalOverlayLibFactory::createModalOverlayLib($libName);
    	return $this->_overlayLibs[$libName];
    }

    public function getInstalledOverlayLibs() {
    	return ModalOverlayLibFactory::getInstalledOverlayLibs();
    }

    
	//
	// Admin
	//
	public function onAdminMenu($menu) {
        array_push($menu,
            array(
                'NAME' => 'Piwishadow',
                'URL' => get_admin_plugin_menu_link(dirname(__FILE__).'/admin/piwishadow_admin.php')
            )
        );
        return $menu;
		
	}

	//
	// Site Event Management
	//
	
	public function onBeginPicture() {
		global $template;
		if ($this->getConfig()->getBool('displayHdPictureInOverlay', 'content') === true) {
			$template->set_prefilter('default_content', 'piwishadow_pictureprefilter');
		}
		if ($this->getConfig()->getBool('displayInOverlay', 'slideshow') === true) {
			$template->set_prefilter('picture', 'piwishadow_slideshowprefilter');
			$template->set_prefilter('slideshow', 'piwishadow_closeslideshowprefilter');
		}
	}
	
	public function onBeginIndex() {
		global $template;
		$test = $this->getConfig();
		
		if ($this->getConfig()->getBool('displayContentInOverlay', 'thumbnail') === true) {
			$template->set_prefilter('index_thumbnails', 'piwishadow_thumbprefilter');
		}
		if ($this->getConfig()->getBool('displayInOverlay', 'slideshow') === true) { 
			$template->set_prefilter('index', 'piwishadow_slideshowprefilter');
			$template->set_prefilter('slideshow', 'piwishadow_closeslideshowprefilter');
		}
	}
	
	protected function assignPluginTemplateVars($template) {
		$c = $this->getConfig()->cursor('thumbnail');
		$o = $this->getConfig()->cursor('overlaylib');
		$template->assign('piwishadow', array(
			'displayMediaPlayerIcon' => $c->get('displayMediaPlayerIcon'),
			'displayContentIcon' => $c->get('displayContentIcon'),
			'displayThumbLabel' => $c->get('displayThumbLabel'),
			'displayContentPageInOverlay' => $c->get('displayContentPageInOverlay'),
			'overlayLibName' => $o->get('modalOverlayLib'),
			'overlayLibPath' => ModalOverlayLibFactory::getOverlayLibPath($o->get('modalOverlayLib')) .DS,
			'pluginPath' => $this->getPluginPath(),
			'pluginUrl' => $this->getPluginUrl()
		));
		
	}
	
	protected function addVideoContentTemplateVars(&$tpl_var, $matches) {
		global $picture;
		// Add piwishadow plugin paramters
		$tpl_var['U_PLAYER'] = $this->getOverlayLib()->getPlayerUrl();
		// TODO: add lib configruation tab in the plugin administration console
		$tpl_var['FLASH_VARS'] = $this->getOverlayLib()->getFlashVarsTemplate();
		
		// Add media parameters
		if (!empty($matches['path']) && !empty($matches['file']) && !empty($matches['media'])) {
			if (isset($tpl_var['NAME'])) $tpl_var['NAME'] = str_replace('_', ' ', $matches['file']);
			$tpl_var['MEDIA'] = strtolower($matches['media']);
			$filePath = ltrim($matches['path'], './');
			$tpl_var['VIDEO_URL'] = get_absolute_root_url(false) . $filePath .'/'. $matches['file'] .'.'. $matches['media'];
			if (!empty($matches['width'])) $tpl_var['VIDEO_WIDTH'] = $matches['width'];
			if (!empty($matches['height'])) $tpl_var['VIDEO_HEIGHT'] = $matches['height'];
		}
	}
	
	public function onEndIndexThumbnails($tpl_thumbnails_var) {
		global $template;
		$pattern = $this->getConfig()->get('mediaFilePattern');
		if (!empty($pattern)) {
			$nbrElements = count($tpl_thumbnails_var);
			for ($i = 0; $i < $nbrElements; $i++) {
				if (preg_match($pattern, $tpl_thumbnails_var[$i]['FILE_PATH'],$matches)) {
					$this->addVideoContentTemplateVars($tpl_thumbnails_var[$i], $matches);
					$tpl_thumbnails_var[$i]['REL_OVERLAY'] = $this->getOverlayLib()->getDefaultLinkRelationship();
					$tpl_thumbnails_var[$i]['U_OVERLAY'] = $tpl_thumbnails_var[$i]['VIDEO_URL'];
				} else { 
					$tpl_thumbnails_var[$i]['REL_OVERLAY'] = $this->getOverlayLib()->getDefaultLinkRelationship();
					$tpl_thumbnails_var[$i]['U_OVERLAY'] = $tpl_thumbnails_var[$i]['FILE_PATH'];
					if ($tpl_thumbnails_var[$i]['FILE_HAS_HD'] === true) {
						$pi = pathinfo($tpl_thumbnails_var[$i]['FILE_PATH']);
						$tpl_thumbnails_var[$i]['U_HIGH'] = $pi['dirname'].'/pwg_high/'.$pi['basename'];
					}
				}
			}
				
			$this->assignPluginTemplateVars($template);
			return $tpl_thumbnails_var;
		}
	}
	
	public function onRenderElementContent($content, $element_info) {
		global $page;
		$pattern = $this->getConfig()->get('mediaFilePattern');
		// Do not display video player in the slideshow
		if (!empty($pattern) && !$page['slideshow']) {
			if (preg_match($pattern, $element_info['image_url'], $matches)) {
				global $page, $template;
				$template->set_template_dir('./plugins/'. basename(dirname(__FILE__)) .'/template');
				$template->set_filenames(
					array('default_content'=>'video_content.tpl')
				);
				$params = array(
			      'SRC_IMG' => $element_info['image_url'],
			      'ALT_IMG' => $element_info['file'],
			      'WIDTH_IMG' => @$element_info['scaled_width'],
			      'HEIGHT_IMG' => @$element_info['scaled_height'],
				);
				$this->addVideoContentTemplateVars($params, $matches);
				$params['REL_OVERLAY'] = $this->getOverlayLib()->getDefaultLinkRelationship();
				$template->assign( $params );
				return $template->parse( 'default_content', true);
			}	
		}
		return $content;
	}
	
	public function onAppendHeader() {
		global $template;
		$piwishadowCSS = '<link rel="stylesheet" href="'. $this->getPluginURL() .'/piwishadow.css" type="text/css" media="screen" />';
		$template->append('head_elements', $piwishadowCSS);
		$this->getOverlayLib()->appendHeaderContent($template, 'head_elements', $this->getConfig()->get('modalOverlayTheme', 'overlaylib'));
	}
} // End class	

$obj = Piwishadow::getInstance($plugin['id']);


// Event admin
add_event_handler('get_admin_plugin_menu_links', array(&$obj, 'onAdminMenu') );

// Event site
add_event_handler('loc_end_page_header', array(&$obj, 'onAppendHeader'));
add_event_handler('loc_begin_picture', array(&$obj, 'onBeginPicture'));
add_event_handler('loc_begin_index', array(&$obj, 'onBeginIndex'));
add_event_handler('loc_end_index_thumbnails', array(&$obj, 'onEndIndexThumbnails'));
if ($obj->getConfig()->getBool('displayMediaPlayer', 'content') === true) {
	// add event handler for rendering element content EVENT_HANDLER_PRIORITY_NEUTRAL - 1 in order to be triggerd before the default handler
	add_event_handler('render_element_content', array(&$obj, 'onRenderElementContent'), EVENT_HANDLER_PRIORITY_NEUTRAL - 1, 2);
}

function piwishadow_pictureprefilter($content, &$smarty) {
	$obj = get_plugin_data(Piwishadow::$PLUGIN_ID);
	$c = $obj->getConfig()->cursor('content');
	return preg_replace($c->get('hdPicturePrefilterPattern'), $c->get('hdPicturePrefilterReplacement'), $content);
}
function piwishadow_slideshowprefilter($content, &$smarty) {
	$obj = get_plugin_data(Piwishadow::$PLUGIN_ID);
	$c = $obj->getConfig()->cursor('slideshow');
	return preg_replace($c->get('prefilterPattern'), $c->get('prefilterReplacement'), $content);
}
function piwishadow_closeslideshowprefilter($content, &$smarty) {
	$obj = get_plugin_data(Piwishadow::$PLUGIN_ID);
	$c = $obj->getConfig()->cursor('slideshow');
	return preg_replace($c->get('closePattern'), $c->get('closeReplacement'), $content);
}
// Prefilter for thumbnail
function piwishadow_thumbprefilter($content, &$smarty) {
	static $thumbButtonsTemplate, $thumbContentTemplate;
	$obj = get_plugin_data(Piwishadow::$PLUGIN_ID);
	$c = $obj->getConfig()->cursor('thumbnail');

	if ($c->getBool('displayContentInOverlay') === true ) {
	
		if (empty($thumbContentTemplate)) {
			$thumbContentTemplate = file_get_contents( $obj->getPluginPath() .DS. 'template' .DS. 'thumbnail_piwishadow_content.tpl' );
		}
		// replace all occurences
		$content = str_replace($c->get('hrefContentSearch'), $thumbContentTemplate, $content);
	}
		
	if ($c->getBool('displayMediaPlayerIcon') === true || $c->getBool('displayContentIcon') === true) {

		if (empty($thumbButtonsTemplate)) {
			$thumbButtonsTemplate = file_get_contents( $obj->getPluginPath() .DS. 'template' .DS. 'thumbnail_piwishadow_buttons.tpl' );
		}
		
		$count = 1; // replace only the first occurence
		$content = str_replace($c->get('buttonsTemplateSearch'), $thumbButtonsTemplate, $content, $count);
	}
	return $content;
}

?>